/* text.js */

// The following comments are for JSLint
/*jslint nomen: true, white: true, vars: true */
/*global document, window, screen, console, M */

// This model creates a G.rand property with a private closure

G.text = ( function () {
	"use strict";

	var PREFIX = "box_";
	var SCALE_FONT = 0.8;
	var DISPLAY_ON = "block";
	var DISPLAY_OFF = "none";
	var EDITOR_FONT_SIZE = 18; // in pixels
	var DEFAULT_EDITOR_WIDTH = 12; // in characters
	var DEFAULT_EDITOR_HEIGHT = 3; // in characters
	var DEFAULT_COLOR = 0x000000;
	var DEFAULT_BG_COLOR = 0xFFFFFF;
	var DEFAULT_ALPHA = 1.0;
	var CURSOR_DEFAULT = "default";
	var CURSOR_POINTER = "pointer";
	var RSHIFT = 256 * 256;
	var GSHIFT = 256;

	// Predefined color strings

	var rstr = [];
	var gstr = [];
	var bstr = [];

	var renderer = null;
	var editor = null;
	var editing = false;
	var count = 0;
	var boxes = {};

	// toRGB( val );
	// val : number = Multiplexed color value
	// Returns DOM rgb string

	var toRGB = function ( rgb ) {
		var red, rval, green, gval, blue, str;

		if ( rgb < 1 ) { // black
			red = 0;
			green = 0;
			blue = 0;
		}
		else if ( rgb >= 0xFFFFFF ) { // white
			red = 0xFF;
			green = 0xFF;
			blue = 0xFF;
		}
		else {
			red = rgb / RSHIFT;
			red = Math.floor( red );
			rval = red * RSHIFT;

			green = ( rgb - rval ) / GSHIFT;
			green = Math.floor( green );
			gval = green * GSHIFT;

			blue = rgb - rval - gval;
		}

		str = rstr[ red ] + gstr[ green ] + bstr[ blue ];
		return str;
	};

	var ePosX = function ( e ) {
		var left;

		left = 0;
		if ( e.offsetParent ) {
			while ( e ) {
				left += e.offsetLeft;
				e = e.offsetParent;
			}
		}
		else if ( e.x ) {
			left += e.x;
		}
		return left;
	};

	var ePosY = function ( e ) {
		var top;

		top = 0;
		if ( e.offsetParent ) {
			while ( e ) {
				top += e.offsetTop;
				e = e.offsetParent;
			}
		}
		else if ( e.y ) {
			top += e.y;
		}
		return top;
	};

	var getCaret = function ( e ) {
		var pos, range;

		pos = 0;
		if ( document.selection ) { // IE
			e.focus();
			range = document.selection.createRange();
			range.moveStart( "character", 0 - e.value.length );
			pos = range.text.length;
		}
		else if ( e.selectionStart || ( e.selectionStart === "0" ) ) {
			pos = e.selectionStart;
		}
		return pos;
	};

	var setCaret = function ( e, pos ) {
		var range;

		if ( e.setSelectionRange ) {
			e.focus();
			e.setSelectionRange( pos, pos );
		}
		else if ( e.createTextRange ) {
			range = e.createTextRange();
			range.collapse( true );
			range.moveEnd( "character", pos );
			range.moveStart( "character", pos );
			range.select();
		}
	};

	var endEvent = function ( event ) {
		if ( event.stopPropagation ) {
			event.stopPropagation();
		}
		else {
			event.cancelBubble = true;
		}
		event.preventDefault(); // prevents weirdness
		return false;
	};

	var getBox = function ( id ) {
		var box = null;

		if ( ( G._type( id ) === "string" ) && ( id.length > 0 ) ) {
			box = boxes[ id ];
			if ( !box ) {
				return null;
			}
		}
		return box;
	};

	var onBoxDown = function ( event ) {
		var id, box, exec;

		id = event.target.dataset.id;
		box = getBox( id );
		if ( box ) {
//            G.text.edit( id, true );

			// This needs fixing

			exec = box.onDown;
			if ( exec && ( typeof exec === "function" ) ) {
				exec();
			}
			else {
				G.text.edit( id, true );
			}
		}
		return endEvent( event );
	};

	var onBoxUp = function ( event ) {
		var id, box, exec;

		id = event.target.dataset.id;
		box = getBox( id );
		if ( box ) {
			//G.monitor.say( "Clicked " + id );
			exec = box.onUp;
			if ( exec && ( typeof exec === "function" ) ) {
				exec();
			}
		}
		return endEvent( event );
	};

	var onBoxOver = function ( event ) {
		var id, box, exec;

		id = event.target.dataset.id;
		box = getBox( id );
		if ( box ) {
			//G.monitor.say( "Clicked " + id );
			exec = box.onOver;
			if ( exec && ( typeof exec === "function" ) ) {
				exec();
			}
		}
		return endEvent( event );
	};

	var onBoxOut = function ( event ) {
		var id, box, exec;

		id = event.target.dataset.id;
		box = getBox( id );
		if ( box ) {
			//G.monitor.say( "Clicked " + id );
			exec = box.onOut;
			if ( exec && ( typeof exec === "function" ) ) {
				exec();
			}
		}
		return endEvent( event );
	};

	var startBox = function ( box ) {

		box.div.addEventListener( "mousedown", onBoxDown, false ); // add onDown handlers
		box.p.addEventListener( "mousedown", onBoxDown, false );

		box.div.addEventListener( "mouseup", onBoxUp, false ); // add onUp handlers
		box.p.addEventListener( "mouseup", onBoxUp, false );

		box.div.addEventListener( "mouseover", onBoxOver, false ); // add onOver handlers
		box.p.addEventListener( "mouseover", onBoxOver, false );

		box.div.addEventListener( "mouseout", onBoxOut, false ); // add onOut handlers
		box.p.addEventListener( "mouseout", onBoxOut, false );

		box.div.style.display = DISPLAY_ON;
		box.div.style.cursor = CURSOR_POINTER;
	};

	var stopBox = function ( box ) {

		box.div.removeEventListener( "mousedown", onBoxDown, false ); // remove onDown handlers
		box.p.removeEventListener( "mousedown", onBoxDown, false );

		box.div.removeEventListener( "mouseup", onBoxUp, false ); // remove onUp handlers
		box.p.removeEventListener( "mouseup", onBoxUp, false );

		box.div.removeEventListener( "mouseover", onBoxOver, false ); // remove onOver handlers
		box.p.removeEventListener( "mouseover", onBoxOver, false );

		box.div.removeEventListener( "mouseout", onBoxOut, false ); // remove onUp handlers
		box.p.removeEventListener( "mouseout", onBoxOut, false );

		box.div.style.cursor = CURSOR_DEFAULT;
		box.div.style.display = DISPLAY_OFF;
	};

	var onInput = function ( event ) {
		var id, box;

		id = editor.dataset.id;
		box = boxes[ id ];
		if ( box ) {
			box.p.innerHTML = box.str = editor.value;
		}
		return true; // else must return true
	};

	var onKeyDown = function ( event ) {
		var key, id, box;

		if ( !event.which ) {
			key = event.keyCode; // IE
		}
		else {
			key = event.which; // Others
		}

		id = editor.dataset.id;
		box = boxes[ id ];
		if ( box ) {
			if ( key === 13 ) {
				box.p.innerHTML = box.str = editor.value;
				stopEditor();
				box.editable = false;
				G.pixi.stopEditing();
				return endEvent( event );
			}
		}
		return true; // else must return true
	};

	var moveEditor = function ( box ) {
		editor.style.left = box.left + box.width + "px";
		editor.style.top = box.top + "px";
	};

	var startEditor = function ( box ) {
		var id, obox;

		id = box.id;
		if ( editing && ( editing !== id ) ) {
			obox = boxes[ editing ];
			obox.editable = false;
		}
		editor.dataset.id = id;
		editor.value = box.str;
		moveEditor( box );
		if ( !editing ) {
			editor.rows = box.lines;
			editor.style.display = DISPLAY_ON; // show textarea
			editor.addEventListener( "input", onInput, false );
			editor.addEventListener( "keydown", onKeyDown, false );
		}
		editing = id;
		editor.focus();
	};

	var stopEditor = function () {
		editing = false;
		editor.removeEventListener( "input", onInput, false );
		editor.removeEventListener( "keydown", onKeyDown, false );
		editor.style.display = DISPLAY_OFF; // hide textarea
	};

	var sys = {

		// G.text.init( r )
		// Initialize the text box API
		// r : object = PIXI renderer

		init : function ( r ) {
			var i;

			renderer = r;

			// Precalculate color string tables

			for ( i = 0; i < 256; i += 1 ) {
				rstr[ i ] = "rgb(" + i + ",";
				gstr[ i ] = i + ",";
				bstr[ i ] = i + ")";
			}

			editor = document.createElement( "textarea" );
			editor.style.display = DISPLAY_OFF;
			editor.style.position = "fixed";
			editor.style.fontFamily = "monospace";
			editor.style.fontSize = EDITOR_FONT_SIZE + "px";
			editor.style.textAlign = "left";
			editor.style.color = "#000000";
			editor.style.backgroundColor = "#FFFFFF";
			editor.style.margin = "0px";
			editor.style.padding = "0px";
			editor.style.border = "1px solid black";
			editor.cols = DEFAULT_EDITOR_WIDTH;
			editor.rows = DEFAULT_EDITOR_HEIGHT;
			editor.style.resize = "none";
			editor.style.overflowY = "scroll";
			editor.style.zIndex = "11";

			renderer.view.parentNode.appendChild( editor );

			editing = false;
		},

		// G.text.create( cnt, options )
		// Creates a new invisible text box
		// (optional) cnt : number = number of text lines ( default = 1 )
		// (optional) options : function | object = function to call when text box is clicked (default = null)
		// If options is an object, any of the following optional properties can be specified:
		// options.onDown : function = function to call when text box is clicked/touched
		// options.onUp : function = function to call when when text box is released
		// options.onOver : function = function to call when when mouse/touch enters text box
		// options.onOut : function = function to call when when mouse/touch leaves text box

		create : function ( cnt, options ) {
			var lines, t, val, down, up, over, out, id, div, p, obj;

			if ( ( cnt !== undefined ) && ( G._type( cnt ) === "number" ) ) {
				lines = Math.floor( cnt );
				if ( lines < 1 ) {
					lines = 1;
				}
			}
			else {
				lines = 1;
			}

			down = out = over = out = null;
			t = G._type( options );
			if ( options ) {
				if ( t === "function" ) {
					down = options;
				}
				else if ( t == "object" ) {
					val = options.onDown;
					if ( val && ( typeof val === "function" ) ) {
						down = val;
					}
					val = options.onUp;
					if ( val && ( typeof val === "function" ) ) {
						up = val;
					}
					val = options.onOver;
					if ( val && ( typeof val === "function" ) ) {
						over = val;
					}
					val = options.onOut;
					if ( val && ( typeof val === "function" ) ) {
						out = val;
					}
				}
			}

			// Create unique id

			count += 1;
			id = PREFIX + count;

			// Create box container <div>

			val = toRGB( DEFAULT_COLOR );
			val = DEFAULT_ALPHA.toString();

			div = document.createElement( "div" );
			div.dataset.id = id;
			div.style.display = DISPLAY_OFF;
			div.style.position = "fixed";
			div.style.zIndex = "10";
			div.style.color = toRGB( DEFAULT_COLOR );
			div.style.backgroundColor = toRGB( DEFAULT_BG_COLOR );
			div.style.opacity = DEFAULT_ALPHA.toString();
			div.style.textAlign = "center";
			div.style.margin = "0 auto";
			div.style.padding = "0";
			div.style.border = "0";
			div.style.overflow = "hidden";
			div.style.cursor = "pointer";

			// Create box paragraph <p>

			p = document.createElement( "p" );
			p.dataset.id = id;
			p.style.position = "relative";
			p.style.display = DISPLAY_ON;
			p.style.fontFamily = "'droid', sans-serif";
			p.style.textAlign = "center";
			p.style.color = "inherit";
			p.style.backgroundColor = "inherit";
			p.style.opacity = "inherit";
			p.style.margin = "0 auto";
			p.style.padding = "0";
			p.style.border = "0";
			p.style.maxWidth = "90%";
			p.style.top = "50%";
			if ( lines === 1 ) {
				p.style.whiteSpace = "nowrap";
			}
			p.style.overflow = "hidden";
			p.style.cursor = "inherit";

			div.appendChild( p ); // add <p> to <div>

			// Add <div> to container of PIXI canvas

			renderer.view.parentNode.appendChild( div );

			// Init box object

			obj = {
				id : id,
				div : div,
				p : p,
				str : "",
				left : 0,
				top : 0,
				width : 0,
				height : 0,
				lines : lines,
				editable : false,
				visible : false,
				onDown : down,
				onUp : up,
				onOver : over,
				onOut : out,
				color : DEFAULT_COLOR,
				bgColor : DEFAULT_BG_COLOR,
				alpha : DEFAULT_ALPHA
			};

			boxes[ id ] = obj;
			return id;
		},

		// G.box.destroy( id )
		// Destroys a previously created box
		// id : string = Box id returned from G.box.create()

		destroy : function ( id ) {
			var box;

			box = getBox( id );
			if ( !box ) {
				return;
			}
			if ( editing === id ) {
				stopEditor();
			}
			if ( box.visible ) {
				stopBox( box );
			}
			box.div.removeChild( box.p );
			renderer.view.parentNode.removeChild( box.div );
			boxes[ id ] = null;
		},

		// G.box.show( id )
		// Show a previously created box
		// id : string = Box id returned from G.box.create()

		show : function ( id ) {
			var box;

			box = getBox( id );
			if ( !box || !box.width || !box.height ) {
				return;
			}
			if ( !box.visible ) {
				box.visible = true;
				startBox( box );
				if ( box.editable ) {
					startEditor( box );
				}
			}
		},

		// G.box.hide( id )
		// Hide a previously created box
		// id : string = Box id returned from G.box.create()

		hide : function ( id ) {
			var box;

			box = getBox( id );
			if ( box && box.visible ) {
				box.visible = false;
				if ( box.editable ) {
					stopEditor();
				}
				stopBox( box );
			}
		},

		// G.box.resize( id, x, y, width, height )
		// Resize a previously created box
		// id : string = Box id returned from G.box.create()
		// x : number = left corner (in pixels) relative to left corner of canvas
		// y : number = top corner (in pixels) relative to top corner of canvas
		// width : number = width of box in pixels
		// height : number = height of box in pixels

		resize : function ( id, x, y, width, height ) {
			var box, left, top, div, p, font_size, margin_top, line_height;

			box = getBox( id );
			if ( !box ) {
				return;
			}

			left = ePosX( renderer.view );
			x += left;
			top = ePosY( renderer.view );
			y += top;

			box.left = x;
			box.top = y;
			box.width = width;
			box.height = height;

			div = box.div;
			div.style.left = x + "px";
			div.style.top = y + "px";
			div.style.width = width + "px";
			div.style.height = height + "px";

			margin_top = "-" + Math.floor( height / 2 ) + "px";
			line_height = Math.floor( height / box.lines ) + "px";
			font_size = Math.floor( ( height / box.lines ) * SCALE_FONT ) + "px";

			p = box.p;
			p.style.marginTop = margin_top;
			p.style.fontSize = font_size;
			p.style.lineHeight = line_height;

			if ( box.visible && box.editable ) {
				moveEditor( box );
			}
		},

		// G.box.set( id, str )
		// Sets the text of a previously created box
		// id : string = Box id returned from G.box.create()
		// str : string = String for box

		set : function ( id, str ) {
			var box;

			box = getBox( id );
			if ( box ) {
				box.p.innerHTML = box.str = str.toString();
			}
		},

		// G.box.get( id )
		// Returns the current text of a previously created box
		// id : string = Box id returned from G.box.create()

		get : function ( id ) {
			var str, box;

			str = "";
			box = getBox( id );
			if ( box ) {
				str = box.str;
			}
			return str;
		},

		// G.text.edit( id, mode )
		// Opens/closes editor for a previously created box
		// id : string = Box id returned from G.box.create()
		// mode : boolean = true to open editor, false to close
		// NOTE 1: Editor closes automatically when RETURN is pressed
		// NOTE 2: Only one editor can be visible at a time

		edit : function ( id, mode ) {
			var box;

			box = getBox( id );

			if ( !box ) {
				return;
			}
			if ( mode && ( G.pixi.myMode() === 2 ) ) {
				if ( !box.editable ) {
					box.editable = true;
					if ( box.visible ) {
						startEditor( box );
					}
				}
				G.pixi.startEditing();
			}
			else {
				if ( box.editable ) {
					box.editable = false;
					if ( box.visible ) {
						stopEditor( box );
					}
				}
				G.pixi.stopEditing();
			}
		},

		// G.text.setColor( id, rgb )
		// Sets the text color of a text box
		// id : string = Box id returned from G.box.create()
		// (optional) rgb : number = multiplexed rgb value (0 to 0xFFFFFF) (default = black)

		setColor : function ( id, rgb ) {
			var box, val;

			box = getBox( id );
			if ( !box ) {
				return;
			}

			val = DEFAULT_COLOR;
			if ( G._type( rgb ) === "number" ) {
				val = Math.floor( rgb );
				if ( val < 0 ) {
					val = 0;
				}
				else if ( val > 0xFFFFFF ) {
					val = 0xFFFFFF;
				}
			}

			box.color = val;
			box.div.style.color = toRGB( val );
		},

		// G.text.setBGColor( id, rgb )
		// Sets the background color of a text box
		// id : string = Box id returned from G.box.create()
		// (optional) rgb : number = multiplexed rgb value (0 to 0xFFFFFF) (default = white)

		setBGColor : function ( id, rgb ) {
			var box, val;

			box = getBox( id );
			if ( !box ) {
				return;
			}

			val = DEFAULT_BG_COLOR;
			if ( G._type( rgb ) === "number" ) {
				val = Math.floor( rgb );
				if ( val < 0 ) {
					val = 0;
				}
				else if ( val > 0xFFFFFF ) {
					val = 0xFFFFFF;
				}
			}

			box.bgColor = val;
			box.div.style.backgroundColor = toRGB( val );
		},

		// G.text.setAlpha( id, alpha )
		// Sets the alpha transparency of a text box
		// id : string = Box id returned from G.box.create()
		// (optional) alpha : number = alpha transparency (0 to 1.0) (default = opaque)

		setAlpha : function ( id, alpha ) {
			var box, val;

			box = getBox( id );
			if ( !box ) {
				return;
			}

			val = DEFAULT_ALPHA;
			if ( G._type( alpha ) === "number" ) {
				val = alpha;
				if ( val < 0 ) {
					val = 0;
				}
				else if ( val > 1 ) {
					val = 1;
				}
			}

			box.opacity = val;
			box.div.style.opacity = val.toString();
		}
	};

	return sys;
}() );
