/* ui.js */

// The following comments are for JSLint
/*jslint nomen: true, white: true, vars: true */
/*global document, window, screen, console, M */

G.ui = ( function () {
	"use strict";
	var windowWidth, windowHeight;



	var system = {
		engine : "GENIAC Simulator",
		major : 1,
		minor : 0,
		revision : 0,
		audio : null, // populated by PS._sys()
		host : {
			app : "",
			version : "",
			os : "" },
		inputs : {
			touch : false
		},
		date : null // populated by init
	};

	// Detect platform and available features

	function systemDetect ()
	{
		var ua, host, browser, os, version;

		ua = window.navigator.userAgent.toLowerCase();
		host = window.navigator.platform.toLowerCase();

		if ( /firefox/.test( ua ) )
		{
			browser = "Firefox";
			if ( /fennec/.test( ua ) )
			{
				browser += " Mobile";
			}
			version = /firefox\/[\.\d]+/.exec( ua )[0].split( '/' )[ 1 ];
		}
		else if ( /chrome/.test( ua ) )
		{
			browser = "Chrome";
			version = /chrome\/[\d\.]+/.exec( ua )[0].split( '/' )[ 1 ];
		}
		else if ( /safari/.test( ua ) )
		{
			browser = 'Safari';
			if ( ( /iphone/.test( ua ) ) || ( /ipad/.test( ua ) ) || ( /ipod/.test( ua ) ) )
			{
				os = 'iOS';
			}
		}
		else if ( /msie/.test( ua ) )
		{
			browser = "Internet Explorer";
			if ( /iemobile/.test( ua ) )
			{
				browser += " Mobile";
			}
			version = /msie \d+[.]\d+/.exec( ua )[0].split( ' ' )[ 1 ];
		}
		else if ( /trident/.test( ua ) ) // IE 11+ on Windows 8
		{
			browser = "Internet Explorer";
			version = /rv\:\d+[.]\d+/.exec( ua )[0].split(':')[1];
		}
		else if ( /opera/.test( ua ) )
		{
			browser = "Opera";
			if ( /mini/.test( ua ) )
			{
				browser += " Mini";
			}
			else if ( /mobile/.test( ua ) )
			{
				browser += " Mobile";
			}
		}
		else if ( /android/.test( ua ) )
		{
			browser = "Android";
			os = /android\s[\.\d]+/.exec( ua );
		}

		if ( !version )
		{
			try
			{
				version = /version\/[\.\d]+/.exec( ua );
				if ( version )
				{
					version = version[0].split( '/' )[ 1 ];
				}
				else
				{
					version = /opera\/[\.\d]+/.exec( ua )[0].split( '/' )[ 1 ];
				}
			}
			catch( err )
			{
				console.error( "Problem detecting browser: " + err.message );
				version = "";
			}
		}

		if ( ( host === "win32" ) || ( host === "win64" ) )
		{
			os = "Windows";
		}
		else if ( ( host === "macintel" ) || ( host === "macppc" ) )
		{
			os = "Mac OS X ";
			os += /10[\.\_\d]+/.exec( ua )[ 0 ];
			if ( /[\_]/.test( os ) )
			{
				os = os.split( '_' ).join( '.' );
			}
		}

		if ( !os ) {
			if ( /linux/.test( host ) )
			{
				os = "Linux";
			}
			else if ( /windows/.test( ua ) )
			{
				os = "Windows";
			}
		}

		system.host.app = browser;
		if ( version ) {
			system.host.version = version;
		}
		system.host.os = os;

	}
	// endEvent( event )
	// Properly terminates a DOM event

	function endEvent ( event )	{
		if ( event.stopPropagation ) {
			event.stopPropagation();
		}
		else {
			event.cancelBubble = true;
		}
		event.preventDefault(); // prevents weirdness
		return false;
	}


	// Any property of <sys> becomes a public property of G.ui

	var sys = {
		// G.ui.init()

		hasTouch : false, // true if touch support exists

		resize : function () {
			var div, menu, menuh, w, h, idealWidth;

			var HEIGHT_PERCENT = 0.9;
			var RATIO = ( 1920 / 1294 );

			windowWidth = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
			windowHeight = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;

			h = Math.floor( windowHeight * HEIGHT_PERCENT );

			idealWidth = Math.floor( h * RATIO );
			if ( idealWidth > windowWidth ) {
				w = windowWidth;
				h = Math.floor( w / RATIO );
			}
			else {
				w = idealWidth;
			}

			menu = document.getElementById( "menubar" );
			menuh = menu.style.height;

			div = document.getElementById( "wrapper" );
			div.style.width = w + "px";

			G.pixi.resize( w, h );
			G.monitor.resize( w );
		},

		// G.ui.mode()
		// str : string = command ID
		// Receives "Wires", "Dials" or "Play"

		mode : function ( str ) {
			G.monitor.show();
			G.monitor.say( str + "\n" );
		},

		// G.ui.clear()
		// str : string = command ID
		// Receives "Item" or "All" or "Monitor"

		clear : function ( str ) {
			G.monitor.show();
			G.monitor.say( str + "\n" );
		},

		// G.ui.project( event )
		// str : string = command ID
		// Receives "Item" or "All"

		project : function ( str ) {
			G.monitor.show();
			G.monitor.say( str + "\n" );
		},

		init : function () {
			var onResize = function ( event ) {
				// calc device dimensions
				G.ui.resize();
				return endEvent( event );
			};

			systemDetect();
			G.ui.hasTouch = Modernizr.touchevents;

			// Set up resize events

			window.addEventListener( "resize", function ( event ) {
				return onResize( event );
			} );

			if ( window.DeviceOrientationEvent ) {
				window.addEventListener( "deviceorientation", function ( event ) {
					return onResize( event );
				} );
			}

			G.ui.resize();
		}
	}

	return sys;
} () );
